////////////////////////////////////////////////////////////////////////////////
//
// (C) 2010 Sarah Eris Horsley Caffee
//
// mandelbrot - A simple Mandelbrot Set viewer.
//
// Fractal.h
// Fractal calculator virtual class definition.  Actual fractals should 
// implement this.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef FRACTAL_H
#define FRACTAL_H

#include <map>
#include <string>

// Need to define this via the pre-processor because if it's a staticaly
// initialized const string we still have no guarantee that it will be 
// initialized before it is needed by a FractalCreator class.
// Do the same in all derived classes.
#define FRACTAL_TYPE_NAME "Fractal"

class Fractal
   {
   public:
   Fractal();

   virtual bool calc_set() = 0;
   virtual bool init(SDL_Surface * surf) = 0;

   double get_re_min();
   double set_re_min(const double r_min);

   double get_im_max();
   double set_im_max(const double i_max);

   double get_size();
   double set_size(const double s);

   unsigned int get_max_iter();
   unsigned int set_max_iter(const unsigned int iter);
   unsigned int inc_max_iter();
   unsigned int dec_max_iter();

   void set_calc_needed();

   virtual const char * get_display_name();
   virtual const char * get_type_name();

   virtual bool get_option(std::string option, void * value) {};
   virtual bool set_option(std::string option, void * value) {};



   protected:

   virtual void draw_pixel(int x, int y, Uint32 * color);
   
   std::string display_name;
   std::string type_name;
   
   double re_min;
   double im_max;
   double size;
   unsigned int max_iter;
   bool calc_needed;
   
   SDL_Surface * surface;
   
   };

class FractalCreator
   {
   public:
   virtual Fractal * create() const = 0;
   };

std::map <std::string, FractalCreator *> & get_Fractal_map();

#endif
