////////////////////////////////////////////////////////////////////////////////
//
// (C) 2010 Sarah Eris Horsley Caffee
//
// mandelbrot - A simple Mandelbrot Set viewer.
//
// Mandelbrot.cpp
// Mandelbrot set calculator class implementation
//
////////////////////////////////////////////////////////////////////////////////

#include <string>

#include "Mandelbrot.h"


////////////////////////////////////////////////////////////////////////////////

Mandelbrot::Mandelbrot()
   {
   display_name = "Mandelbrot set";
   type_name = MANDELBROT_TYPE_NAME;
   re_min = -2.5;
   im_max = 2.5;
   size = 5.0;
   max_iter = 50;
   calc_needed = true;
   }


////////////////////////////////////////////////////////////////////////////////
bool Mandelbrot::init(SDL_Surface * surf)
   {
   if (!surf)
      {
      return false;
      }
   else
      {
      surface = surf;
      }
   calc_needed = true;
   return true;
   }


////////////////////////////////////////////////////////////////////////////////

bool Mandelbrot::calc_set()
   {
   if (!surface) return false;

   // Code from http://warp.povusers.org/Mandelbrot/

   if (calc_needed)
      {
      double re_scale = (size)/(surface->w-1);
      double im_scale = (size)/(surface->h-1);

      int r, g, b;

      Uint32 black = SDL_MapRGB(surface->format, 0, 0, 0);
      Uint32 color;
      int halfway = max_iter/2 - 1;
      float step = (float) 255 / halfway;

      unsigned x, y, n;

      for(y = 0; y < surface->h; ++y)
	 {
	 double c_im = im_max - y * im_scale;
	 for(x = 0; x < surface->w; ++x)
	    {
	    double c_re = re_min + x * re_scale;

	    double z_re = c_re, z_im = c_im;
	    bool in_set = true;
	    for(n=0; n < max_iter; ++n)
	       {
	       double z_re2 = z_re * z_re, z_im2 = z_im * z_im;
	       if(z_re2 + z_im2 > 4)
		  {
		  in_set = false;
		  break;
		  }
	       z_im = 2 * z_re * z_im + c_im;
	       z_re = z_re2 - z_im2 + c_re;
	       }
	    if(in_set) 
	       { 
	       draw_pixel(x, y, &black); 
	       }
	    else
	       {
	       if (n <= halfway)
		  {
		  b = n * step;
		  r = g = 0;
		  }
	       else 
		  {
		  b = 255;
		  r = g = (n - halfway) * step;
		  }
	       color = SDL_MapRGB(surface->format, r, g, b);
	       draw_pixel(x, y, &color); 
	       }
	    }
	 }
      }

   calc_needed = false;
   return true;
   }


////////////////////////////////////////////////////////////////////////////////
const char * Mandelbrot::get_type_name()
   {
   return MANDELBROT_TYPE_NAME;
   }


////////////////////////////////////////////////////////////////////////////////

void Mandelbrot::draw_pixel(int x, int y, Uint32 * color)
   {
   Fractal::draw_pixel(x, y, color);
   }


